"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SyntaxHighlighter = void 0;
var _core = require("@shikijs/core");
var _engineJavascript = require("@shikijs/engine-javascript");
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
let highlighterInstance = null;
const loadedLanguages = new Set();
async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await (0, _core.createHighlighterCore)({
      engine: (0, _engineJavascript.createJavaScriptRegexEngine)(),
      themes: [],
      langs: []
    });
  }
  return highlighterInstance;
}
async function ensureLanguageLoaded(highlighter, language) {
  const langId = typeof language === 'string' ? language : language.name;
  if (!langId) {
    throw new Error('Language must have an id or name');
  }
  if (!loadedLanguages.has(langId)) {
    if (typeof language === 'string') {
      // For string identifiers, we need to load the language
      const langInfo = {
        scopeName: `source.${language}`,
        aliases: [language],
        name: language,
        repository: {},
        patterns: []
      };
      await highlighter.loadLanguage(langInfo);
    } else {
      // For LanguageRegistration objects, we use them directly
      await highlighter.loadLanguage(language);
    }
    loadedLanguages.add(langId);
  }
  return langId;
}
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  debugMode = false
}) => {
  const [tokens, setTokens] = _react.default.useState([]);
  const [loading, setLoading] = _react.default.useState(true);
  const [error, setError] = _react.default.useState(null);
  _react.default.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true);
        const highlighter = await getHighlighter();
        if (debugMode) {
          // eslint-disable-next-line no-console
          console.log('Highlighter initialization state:', {
            loadedLanguages: Array.from(loadedLanguages),
            inputText: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
            languageType: typeof language === 'string' ? 'string' : 'LanguageRegistration',
            languageValue: typeof language === 'string' ? language : language.name
          });
        }

        // Ensure language is loaded
        const langId = await ensureLanguageLoaded(highlighter, language);
        if (debugMode) {
          // eslint-disable-next-line no-console
          console.log('Language loaded:', {
            id: langId,
            loadedLanguages: Array.from(loadedLanguages)
          });
        }

        // Ensure theme has a name property
        const themeName = typeof theme === 'string' ? theme : theme.name;
        if (!themeName) {
          throw new Error('Theme must have a name property');
        }
        await highlighter.loadTheme(theme);

        // Debug logging for theme details
        if (debugMode) {
          const loadedTheme = highlighter.getTheme(themeName);
          // eslint-disable-next-line no-console
          console.log('Theme and scope details:', {
            theme: themeName,
            scopeName: typeof language === 'string' ? `source.${language}` : language.scopeName,
            settings: loadedTheme.settings?.map(s => ({
              scope: s.scope,
              foreground: s.settings?.foreground,
              fontStyle: s.settings?.fontStyle
            }))
          });
        }

        // Highlight with explanations enabled
        const result = highlighter.codeToTokens(text, {
          lang: langId,
          theme: themeName,
          includeExplanation: true
        });
        if (debugMode && result.tokens[0]) {
          // eslint-disable-next-line no-console
          console.log('First line token analysis:', result.tokens[0].map(token => ({
            content: token.content,
            color: token.color,
            scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName)
          })));
        }
        setTokens(result.tokens);
        setError(null);
      } catch (err) {
        if (debugMode) {
          console.error('Highlighting error:', {
            error: err,
            message: err instanceof Error ? err.message : 'Unknown error',
            state: {
              text: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
              language,
              loadedLanguages: Array.from(loadedLanguages)
            }
          });
        }
        setError(err instanceof Error ? err.message : 'Failed to highlight code');
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [text, language, theme, debugMode]);
  const renderToken = (token, index) => {
    let color = token.color;
    if (!color && token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        if (scope.themeMatches?.[0]?.settings.foreground) {
          color = scope.themeMatches[0].settings.foreground;
          break;
        }
      }
    }
    const defaultColor = theme.colors?.['editor.foreground'] || '#e1e4e8';
    const tokenStyle = {
      fontSize,
      color: color || defaultColor,
      ...(token.fontStyle === 1 && {
        fontStyle: 'italic'
      }),
      ...(token.fontStyle === 2 && {
        fontWeight: 'bold'
      }),
      ...(token.fontStyle === 4 && {
        textDecorationLine: 'underline'
      })
    };
    const style = _reactNative.StyleSheet.flatten([textStyle, tokenStyle]);
    if (debugMode) {
      // eslint-disable-next-line no-console
      console.log('Token details:', {
        content: token.content,
        scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName),
        color,
        style
      });
    }
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.Text, {
      style: style,
      children: token.content
    }, `${token.offset}-${index}`);
  };
  const renderLine = (lineTokens, index) => /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
    style: {
      flexDirection: 'row',
      flexWrap: 'wrap'
    },
    children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
  }, index);
  if (loading) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        backgroundColor: theme.colors?.['editor.background'],
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        backgroundColor: theme.colors?.['editor.background'],
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsxs)(_reactNative.Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor: theme.colors?.['editor.background']
    }, scrollViewProps?.style],
    contentContainerStyle: [containerStyle],
    children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
exports.SyntaxHighlighter = SyntaxHighlighter;
var _default = exports.default = SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map